// kgimp.c
// Klein Goldon Reterted Propagation
// Impulse Response
// Copyright (C) 1994,2001 Yukinori NAGATANI
// nagatani@yukawa.kyoto-u.ac.jp

#include <stdio.h>
#include <stdlib.h>

#include "wav.h"
#include "bessel1.h"

class KleinGoldonInpulse : public RealWave {
public:
	Real	rMass;
	Real	rX;
	KleinGoldonInpulse (size_t size = 0) : RealWave (size) {
		rMass = 400.0 * (2 * M_PI);	// m = 400 Hz
		rX = 10.0;			// distance = 1 sec
	}
	void showParameter (void) {
		fprintf (stderr,
			 "Time = %3.1lf (sec)  ",
			 (Real) size / Header.sample_fq);
		fprintf (stderr,
			 "Mass = %6.1lf (Hz)  ", rMass / (2*M_PI));
		fprintf (stderr,
			 "distance = %3.1lf (sec)\n", rX);
	}
	void create () {
		malloc ();
		Real	rDTime = 1.0 / Header.sample_fq;
		for (size_t n = 0 ; n < size ; n++)
			prBuffer [n]
			= rGreenFunctionMulMass (rDTime*n + rX, rX, rMass);
		sizeRec = size;
	}
	static Real rGreenFunctionMulMass (Real t, Real x, Real rMass)
	{
		Real	l,	ll;
		
		if (0.0 < (ll = t*t - x*x)) {
			l = sqrt( ll);
			return	- rBessel1 (rMass * l) * x/l;
		} else
			return	0.0;
	}
};

#ifdef KGIMP_MAIN
const char * szTitleMessage =
	"Klein-Goldon Impulse Creater  by Y.Nagatani";

const char * szHelpMessage =
	"Usage: kgimp [-m <400:mass (Hz)>]\n"
	"             [-x <10:distance (sec)>]\n"
	"             [-T <3:timesize (sec)>]";

void error (const char * szMessage) {
	fprintf (stderr, "kgimp:%s\n%s\n", szMessage, szHelpMessage);
	exit (1);
}

void main (int nArgc, char *pszArgv[])
{
	KleinGoldonInpulse	Impulse;
	Impulse.size = 44100 * 3;
	
	nArgc--;
	pszArgv++;
	for (; nArgc > 0 ; nArgc--, pszArgv++) {
		if (!strcmp (*pszArgv, "-m")) {
			nArgc--;
			pszArgv++;
			if (nArgc <= 0)
				error ("no args for -m");
			Impulse.rMass = atof (*pszArgv) * (2 * M_PI);
			continue;
		}
		if (!strcmp (*pszArgv, "-x")) {
			nArgc--;
			pszArgv++;
			if (nArgc <= 0)
				error ("no args for -x");
			Impulse.rX = atof (*pszArgv);
			continue;
		}
		if (!strcmp (*pszArgv, "-T")) {
			nArgc--;
			pszArgv++;
			if (nArgc <= 0)
				error ("no args for -T");
			Impulse.size = (int) (44100 * atof (*pszArgv));
			continue;
		}
		error ("Invalid Argument");
	}
	fprintf (stderr, "%s\n", szTitleMessage);
	
	Impulse.showParameter ();
	Impulse.create ();
	Impulse.write ();
}
#endif
